#!/usr/bin/perl

######################################################################
package Cognition::uF::hFreebusy;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Digest::SHA1 qw(sha1_hex);

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub MAX_INCLUDE_ITERATIONS { 4; }


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = Cognition::HTMLParser::Utils::searchClass('vfreebusy', $within);
	my @rv;
	
	foreach my $ev (@nodes)
	{
		push @rv, parse($page, $ev);
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	if (length $this->{uid})
	{
		my $uid = $this->{uid};
		$uid = "urn:x-uid:$uid"
			unless ($uid =~ /^[a-z0-9]{1,10}\:/i);
		push @rv, $uid;
	}
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
#	if (length $this->{summary})
#	{
#		push @rv, 'urn:x-vfreebusy:' . CGI::Util::escape($this->{summary});
#	}
#	push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	push @rv, Cognition::HTMLParser::Utils::BNodeURI($this->{_dom}, 'FreeBusy')
		unless (@rv);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:2445#';

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new($ns.'Vfreebusy', 1));
	
	# Patterns for simple data.
	my @_SINGLES  = qw(summary comment dtstart dtend duration dtstamp);
	my @_SINGLESR = qw(uid parent);
	my @_MULTIS   = qw();
	my @_MULTISR  = qw(url child sibling);
	my @_OBJECTS  = qw(freebusy contact attendee organizer);
	
	# Properties which exist as singles.
	foreach my $property (@_SINGLES)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property});
		$subject->addVal($ns.$property, $val);
	}

	# Resource-type properties which exist as singles.
	foreach my $property (@_SINGLESR)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property}, 1);
		$subject->addVal($ns.$property, $val);
	}
		
	# Properties which exist as multiple values.
	foreach my $property (@_MULTIS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V);
			$subject->addVal($ns.$property, $val);
		}
	}
	
	# Resource-type properties which exist as multiple values.
	foreach my $property (@_MULTISR)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V, 1);
			$subject->addVal($ns.$property, $val);
		}
	}

	# Embedded objects which may be represented as resources
	foreach my $property (@_OBJECTS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			my $val;
			if (ref $V)
				{ $val = Cognition::RDFModel::Value->new($V->uri, 1); }
			else
				{ $val = Cognition::RDFModel::Value->new($V); }
			$subject->addVal($ns.$property, $val);
		}
	}
	
	return $subject;
	
} #/sub rdf_subject


sub metadata_ns
{
	my $this = shift;
	return { 'ical' => { nsurl=>'urn:ietf:rfc:2445#',title=>'iCalendar' } };
} #/sub metadata_ns


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $event_node = $rv->{'_dom'}->cloneNode(1);
	
	my $id = $event_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($event_node, $page) 
		              + Cognition::uF::ufp_include2($event_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($event_node, $page);
	Cognition::uF::ufp_table($event_node, $page);

	# Check nested hcards to see if they have a 'useful' class, and if so, parse them in.
	my @nested = Cognition::HTMLParser::Utils::searchClass('vcard', $event_node);
	foreach my $h (@nested)
	{
		if ($h->getAttribute('class') =~ / (^|\s) organizer (\s|$) /x)
		{
			push @{ $rv->{organizer} }, Cognition::uF::hCard::parse($page, $h);
		}
		
		if ($h->getAttribute('class') =~ / (^|\s) contact (\s|$) /x)
		{
			push @{ $rv->{contact} }, Cognition::uF::hCard::parse($page, $h);
		}
		
		if ($h->getAttribute('class') =~ / (^|\s) attendee (\s|$) /x)
		{
			push @{ $rv->{attendee} }, Cognition::uF::hCard::parse($page, $h);
		}
		
		my $newClass = $h->getAttribute('class');
		$newClass =~ s/\b(organizer|contact|attendee)\b//g;
		$h->setAttribute('class', $newClass);
	}

	# Now that we have reached here, no other composite microformats have
	# any business being nested within this root element. So in the interest
	# of Microformat opacity, let's destroy them.
	Cognition::uF::destroyer($event_node);	
	
	# Simple attributes - singles
	my @simples = qw(summary comment dtstart dtend duration dtstamp uid url);
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $event_node);
		my $value_node  = $value_nodes[0];
		next unless ($value_node);
		
		my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node, 'value');

		if ($simple =~ '/^u/')
		{
			if ($simple eq 'uid' && length $value_node->getAttribute('id'))
				{ $value = $page->abs_url . '#' . $value_node->getAttribute('id'); }
			elsif ($simple eq 'uid' && ($value_node->localname eq 'a' || $value_node->localname eq 'area') && length $value_node->getAttribute('name'))
				{ $value = $page->abs_url . '#' . $value_node->getAttribute('name'); }
			elsif ($value_node->localname eq 'a' || $value_node->localname eq 'area')
				{ $value = $page->abs_url($value_node->getAttribute('href')); }
			elsif ($value_node->localname eq 'img')
				{ $value = $page->abs_url($value_node->getAttribute('src')); }
			elsif ($value_node->localname eq 'object')
				{ $value = $page->abs_url($value_node->getAttribute('data')); }
		}

		if ($simple =~ /^(dtstart|dtend|dtstamp)$/ && length $value)
		{
			$value = Cognition::uF::datetime($value);
			$value->subtract(seconds=>1)
				if ($value_node->getAttribute('class') =~ /beforethis/i && $simple eq 'dtend');
		}
			
		$rv->{$simple} = $value;
	}	

	# Simple attributes - possible duplicates
	my @simples = qw(contact attendee organizer);	
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $event_node);
		foreach my $value_node (@value_nodes)
		{
			next unless ($value_node);
			
			my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node, 'value');
			
			if ($simple =~ /^(url)$/)
			{
				if ($value_node->localname eq 'a' || $value_node->localname eq 'area')
					{ $value = $page->abs_url($value_node->getAttribute('href')); }
				elsif ($value_node->localname eq 'img')
					{ $value = $page->abs_url($value_node->getAttribute('src')); }
				elsif ($value_node->localname eq 'object')
					{ $value = $page->abs_url($value_node->getAttribute('data')); }
			}
			push @{$rv->{$simple}}, $value;
		}
	}	
	
	# Freebusy itself
	my @nodes = Cognition::HTMLParser::Utils::searchClass('freebusy', $event_node);
	foreach my $n (@nodes)
	{
		my @accumValue;
		my @accumType;
		
		foreach my $v (Cognition::HTMLParser::Utils::searchClass('value', $n))
		{
			push @accumValue, Cognition::HTMLParser::Utils::STRINGIFY($v, undef, 1);
			$v->getParentNode->removeChild($v);
		}
		
		my $type;
		my @nodes = Cognition::HTMLParser::Utils::searchClass('fbtype', $n);
		$type = uc(Cognition::HTMLParser::Utils::STRINGIFY($nodes[0], undef, 1)) if (@nodes);
		
		my $C = {
				'value'  => \@accumValue,
				'fbtype' => $type,
				'_id'    => $n->getAttribute('id'),
				'_xp'    => Cognition::HTMLParser::Utils::XPointer($n, $page->abs_url),
				'_page'  => $page,
				'_component' => $rv,
				'_dom'   => $n
			};
		bless $C, "Cognition::uF::hFreebusy::fb";
		push @{ $rv->{freebusy} }, $C;
	}
	
	
	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::hFreebusy






######################################################################
package Cognition::uF::hFreebusy::fb;
######################################################################

sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;

	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	push @rv, Cognition::HTMLParser::Utils::BNodeURI($this->{_dom}, 'FBInfo');
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri

sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:2445#';

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new($ns.'Freebusy', 1));
	
	# Patterns for simple data.
	my @_SINGLES  = qw(fbtype);
	my @_MULTIS   = qw(value);
	
	# Properties which exist as singles.
	foreach my $property (@_SINGLES)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property});
		$subject->addVal($ns.$property, $val);
	}

	# Properties which exist as multiple values.
	foreach my $property (@_MULTIS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V);
			$subject->addVal($ns.$property, $val);
		}
	}
	
	return $subject;
	
} #/sub rdf_subject

1; #/package Cognition::uF::hFreebusy::fb
